@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string>
 #include <vector>
 #include <stdint.h>
+#include <map>
 
 enum class Sensitivity { FAST = 0, MID_SENSITIVE = 1, SENSITIVE = 2, MORE_SENSITIVE = 3, VERY_SENSITIVE = 4, ULTRA_SENSITIVE = 5 };
 enum class TracebackMode { NONE = 0, SCORE_ONLY = 1, STAT = 2, VECTOR = 3, SCORE_BUFFER = 4 };
@@ -100,7 +101,6 @@ struct Config
 	unsigned target_fetch_size;
 	bool mode_more_sensitive;
 	string matrix_file;
-	double lambda, K;
 	string_vector shape_mask;
 	unsigned query_gencode;
 	string unaligned;
@@ -157,7 +157,6 @@ struct Config
 	double freq_treshold;
 	bool use_lazy_dict;
 	string aligned_file;
-	int filter_locus;
 	bool use_dataset_field;
 	bool store_query_quality;
 	string invocation;
@@ -238,7 +237,6 @@ struct Config
 	double ranking_score_drop_factor;
 	double ranking_cutoff_bitscore;
 	int left_most_interval;
-	bool alp;
 	bool no_forward_fp;
 	bool no_ref_masking;
 	string roc_file;
@@ -247,6 +245,13 @@ struct Config
 	bool output_fp;
 	int family_cap;
 	int cbs_matrix_scale;
+	size_t query_count;
+	double cbs_angle;
+	int target_seg;
+	double cbs_err_tolerance;
+	int cbs_it_limit;
+	double query_match_distance_threshold;
+	double length_ratio_threshold;
 
 	Sensitivity sensitivity;
 	TracebackMode traceback_mode;
@@ -266,10 +271,27 @@ struct Config
 	int algo;
 
 	string cluster_algo;
+	string	cluster_similarity;
+
+	size_t max_size_set;
+	string_vector cluster_steps;
 	double cluster_mcl_inflation;
 	uint32_t cluster_mcl_expansion;
 	double cluster_mcl_sparsity_switch;
-	string	cluster_similarity;
+	uint32_t cluster_mcl_chunk_size;
+	uint32_t cluster_mcl_max_iter;
+	string	cluster_mcl_graph_file;
+	bool	cluster_mcl_restart;
+
+	
+	std::map<std::string, Sensitivity> sens_map{
+		{"fast", Sensitivity::FAST},
+		{"sensitive", Sensitivity::SENSITIVE},
+		{"mid-sensitive", Sensitivity::MID_SENSITIVE},
+		{"more-sensitive", Sensitivity::MORE_SENSITIVE},
+		{"very-sensitive", Sensitivity::VERY_SENSITIVE},
+		{"ultra-sensitive", Sensitivity::ULTRA_SENSITIVE}
+	};
 
 	enum { query_parallel = 0, target_parallel = 1 };
 	unsigned load_balancing;
@@ -313,4 +335,4 @@ extern Config config;
 template<typename _t>
 _t top_cutoff_score(_t top_score) {
 	return _t((1.0 - config.toppercent / 100.0)*top_score);
-}
\ No newline at end of file
+}
