@@ -35,41 +35,65 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../dp/dp.h"
 #include "../basic/masking.h"
 #include "cluster.h"
+#include <unordered_map>
+#include <numeric>
 
 using namespace std;
 
 namespace Workflow { namespace Cluster{ 
 
+	struct NodEdgSet {
+		uint32_t nodes;
+		size_t edges;
+		uint32_t set;
+	};
+
 class MultiStep : public ClusteringAlgorithm {
 private:
 	BitVector rep_bitset(const vector<int> &centroid, const BitVector *superset = nullptr);
-	vector<int> cluster(DatabaseFile &db, const BitVector *filter);
+	vector<int> cluster(DatabaseFile& db, const BitVector* filter);
+	unordered_map<uint32_t, NodEdgSet> find_connected_components(vector<uint32_t>& sindex, const vector<size_t>& nedges);
+	void mapping_comp_set(unordered_map<uint32_t, NodEdgSet>& comp);
+	void steps(BitVector& current_reps, BitVector& previous_reps, vector<int>& current_centroids, vector<int>& previous_centroids, int count);
+
 public:
+	~MultiStep(){};
 	void run();
-	string get_key();
 	string get_description();
+	static string get_key(){
+		return "multi-step";
+	}
 };
 
 struct Neighbors : public vector<vector<int>>, public Consumer {
-	Neighbors(size_t n):
-		vector<vector<int>>(n)
-	{}
-	virtual void consume(const char *ptr, size_t n) override {
-		int query, subject, count;
-		float qcov, scov, bitscore;
-		const char *end = ptr + n;
+	Neighbors(size_t n) : vector<vector<int>>(n), smallest_index(n,0), number_edges(n,0){
+		iota(smallest_index.begin(), smallest_index.end(), 0);
+	}
+
+	vector<uint32_t> smallest_index;
+	vector<size_t> number_edges;
+	
+	
+	virtual void consume(const char* ptr, size_t n) override {
+		const char* end = ptr + n;
+
 		while (ptr < end) {
-			if (sscanf(ptr, "%i\t%i\t%f\t%f\t%f\n%n", &query, &subject, &qcov, &scov, &bitscore, &count) != 5)
-				throw runtime_error("Cluster format error.");
-			ptr += count;
-			if (query == subject)
-				continue;
-			//cout << query << '\t' << subject << '\t' << qcov << '\t' << scov << '\t' << endl;
+			const uint32_t query = *(uint32_t*)ptr;
+			ptr += sizeof(uint32_t);
+			const uint32_t subject = *(uint32_t*)ptr;
+			ptr += sizeof(uint32_t);
+
 			(*this)[query].push_back(subject);
-			edges.push_back({ query, subject, (int)bitscore });
+
+			if (subject < smallest_index[query])
+				smallest_index[query] = subject;
+
+			if (query < smallest_index[subject])
+				smallest_index[subject] = query;
+
+			++number_edges[query];
 		}
 	}
-	vector<Util::Algo::Edge> edges;
+	
 };
-
 }}
