@@ -43,11 +43,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../util/io/record_reader.h"
 #include "../util/parallel/multiprocessing.h"
 
-String_set<char, '\0'>* ref_ids::data_ = 0;
+String_set<char, '\0'>* ref_ids::data_ = nullptr;
 Partitioned_histogram ref_hst;
 unsigned current_ref_block;
-Sequence_set* ref_seqs::data_ = 0;
+Sequence_set* ref_seqs::data_ = nullptr;
 bool blocked_processing;
+std::vector<uint32_t> block_to_database_id;
 
 using namespace std;
 
@@ -312,8 +313,7 @@ void DatabaseFile::seek_direct() {
 	seek(sizeof(ReferenceHeader) + sizeof(ReferenceHeader2) + 8);
 }
 
-bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, const size_t max_letters,
-	Sequence_set **dst_seq, String_set<char, 0> **dst_id, bool load_ids, const vector<bool> *filter, const bool fetch_seqs, const Chunk & chunk)
+bool DatabaseFile::load_seqs(vector<uint32_t>* block2db_id, const size_t max_letters, Sequence_set **dst_seq, String_set<char, 0> **dst_id, bool load_ids, const BitVector* filter, const bool fetch_seqs, const Chunk & chunk)
 {
 	task_timer timer("Loading reference sequences");
 
@@ -328,7 +328,7 @@ bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, const size_
 	size_t letters = 0, seqs = 0, id_letters = 0, seqs_processed = 0, filtered_seq_count = 0;
 	vector<uint64_t> filtered_pos;
 	vector<bool> filtered_seqs;
-	block_to_database_id.clear();
+	if (block2db_id) block2db_id->clear();
 
 	if (fetch_seqs) {
 		*dst_seq = new Sequence_set;
@@ -351,7 +351,7 @@ bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, const size_
 	while (goon()) {
 		Pos_record r_next;
 		(*this) >> r_next;
-		if (!filter || (*filter)[database_id]) {
+		if (!filter || filter->get(database_id)) {
 			letters += r.seq_len;
 			if (fetch_seqs) {
 				(*dst_seq)->reserve(r.seq_len);
@@ -363,7 +363,7 @@ bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, const size_
 			}
 			//++seqs;
 			++filtered_seq_count;
-			block_to_database_id.push_back((unsigned)database_id);
+			if (block2db_id) block2db_id->push_back((unsigned)database_id);
 			if (filter) {
 				filtered_pos.push_back(last ? 0 : r.pos);
 				filtered_seqs.push_back(true);
@@ -397,22 +397,20 @@ bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, const size_
 		if(load_ids) (*dst_id)->finish_reserve();
 		seek(start_offset);
 
-		size_t n = 0;
-		for (size_t i = 0; i < seqs; ++i) {
-			//if (filter && filtered_pos[n]) seek(filtered_pos[n]);
-			if (filter && !filtered_seqs[i]) {
+		for (size_t i = 0; i < filtered_seq_count; ++i) {
+			if (filter && filtered_pos[i]) seek(filtered_pos[i]);
+			/*if (filter && !filtered_seqs[i]) {
 				skip_seq();
 				continue;
-			}
-			read((*dst_seq)->ptr(n) - 1, (*dst_seq)->length(n) + 2);
-			*((*dst_seq)->ptr(n) - 1) = sequence::DELIMITER;
-			*((*dst_seq)->ptr(n) + (*dst_seq)->length(n)) = sequence::DELIMITER;
+			}*/
+			read((*dst_seq)->ptr(i) - 1, (*dst_seq)->length(i) + 2);
+			*((*dst_seq)->ptr(i) - 1) = sequence::DELIMITER;
+			*((*dst_seq)->ptr(i) + (*dst_seq)->length(i)) = sequence::DELIMITER;
 			if (load_ids)
-				read((*dst_id)->ptr(n), (*dst_id)->length(n) + 1);
+				read((*dst_id)->ptr(i), (*dst_id)->length(i) + 1);
 			else
 				if (!seek_forward('\0')) throw std::runtime_error("Unexpected end of file.");
-			Masking::get().remove_bit_mask((*dst_seq)->ptr(n), (*dst_seq)->length(n));
-			++n;
+			Masking::get().remove_bit_mask((*dst_seq)->ptr(i), (*dst_seq)->length(i));
 		}
 		timer.finish();
 		(*dst_seq)->print_stats();
@@ -430,6 +428,8 @@ void DatabaseFile::read_seq(string &id, vector<Letter> &seq)
 {
 	char c;
 	read(&c, 1);
+	seq.clear();
+	id.clear();
 	read_to(std::back_inserter(seq), '\xff');
 	read_to(std::back_inserter(id), '\0');
 }
