@@ -19,7 +19,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef SEQUENCE_SET_H_
 #define SEQUENCE_SET_H_
 
-#include <iostream>
 #include <string>
 #include <algorithm>
 #include <queue>
@@ -31,10 +30,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../util/ptr_vector.h"
 #include "../basic/value.h"
 
-using std::cout;
-using std::endl;
-using std::pair;
-
 struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 {
 
@@ -43,7 +38,7 @@ struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 	
 	void print_stats() const
 	{
-		verbose_stream << "Sequences = " << this->get_length() << ", letters = " << this->letters() << ", average length = " << this->avg_len() << endl;
+		verbose_stream << "Sequences = " << this->get_length() << ", letters = " << this->letters() << ", average length = " << this->avg_len() << std::endl;
 	}
 
 	sequence operator[](size_t i) const
@@ -51,7 +46,7 @@ struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 		return sequence(ptr(i), length(i));
 	}
 
-	pair<size_t, size_t> len_bounds(size_t min_len) const
+	std::pair<size_t, size_t> len_bounds(size_t min_len) const
 	{
 		const size_t l(this->get_length());
 		size_t max = 0, min = std::numeric_limits<size_t>::max();
@@ -59,7 +54,7 @@ struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 			max = std::max(this->length(i), max);
 			min = this->length(i) >= min_len ? std::min(this->length(i), min) : min;
 		}
-		return pair<size_t, size_t>(min, max);
+		return std::pair<size_t, size_t>(min, max);
 	}
 
 	size_t max_len(size_t begin, size_t end) const
@@ -70,9 +65,9 @@ struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 		return max;
 	}
 
-	vector<size_t> partition(unsigned n_part) const
+	std::vector<size_t> partition(unsigned n_part) const
 	{
-		vector<size_t> v;
+		std::vector<size_t> v;
 		const size_t l = (this->letters() + n_part - 1) / n_part;
 		v.push_back(0);
 		for (unsigned i = 0; i < this->get_length();) {
@@ -111,7 +106,7 @@ struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 	}
 
 	template <typename _f, typename _filter>
-	void enum_seeds(PtrVector<_f> &f, const vector<size_t> &p, size_t shape_begin, size_t shape_end, const _filter *filter, bool contig = false) const
+	void enum_seeds(PtrVector<_f> &f, const std::vector<size_t> &p, size_t shape_begin, size_t shape_end, const _filter *filter, bool contig = false) const
 	{
 		std::vector<std::thread> threads;
 		for (unsigned i = 0; i < f.size(); ++i)
@@ -126,7 +121,7 @@ struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 private:
 
 	template<typename _f, typename _filter>
-	void enum_seeds(_f *f, unsigned begin, unsigned end, pair<size_t, size_t> shape_range, const _filter *filter) const
+	void enum_seeds(_f *f, unsigned begin, unsigned end, std::pair<size_t, size_t> shape_range, const _filter *filter) const
 	{
 		vector<Letter> buf(max_len(begin, end));
 		uint64_t key;
@@ -150,7 +145,7 @@ private:
 	}
 
 	template<typename _f, uint64_t _b, typename _filter>
-	void enum_seeds_hashed(_f *f, unsigned begin, unsigned end, pair<size_t, size_t> shape_range, const _filter *filter) const
+	void enum_seeds_hashed(_f *f, unsigned begin, unsigned end, std::pair<size_t, size_t> shape_range, const _filter *filter) const
 	{
 		uint64_t key;
 		for (unsigned i = begin; i < end; ++i) {
@@ -193,7 +188,7 @@ private:
 	}
 
 	template<typename _f, typename _filter>
-	static void enum_seeds_worker(_f *f, const Sequence_set *seqs, unsigned begin, unsigned end, pair<size_t,size_t> shape_range, const _filter *filter, bool contig)
+	static void enum_seeds_worker(_f *f, const Sequence_set *seqs, unsigned begin, unsigned end, std::pair<size_t,size_t> shape_range, const _filter *filter, bool contig)
 	{
 		static const char *errmsg = "Unsupported contiguous seed.";
 		if (shape_range.second - shape_range.first == 1 && shapes[shape_range.first].contiguous() && shapes.count() == 1 && (config.algo == Config::query_indexed || contig)) {
