@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,18 +20,20 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef SYSTEM_H_
-#define SYSTEM_H_
-
+#pragma once
 #include <stdexcept>
 
 #ifdef _MSC_VER
 
 #define PACKED_ATTRIBUTE
+#define FORCE_INLINE __forceinline
+#define MSC_INLINE __forceinline
 
 #else
 
 #define PACKED_ATTRIBUTE __attribute__((packed))
+#define FORCE_INLINE __attribute__((always_inline))
+#define MSC_INLINE
 
 #endif
 
@@ -39,4 +45,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define POSIX_OPEN2(x,y) open(x,y)
 #endif
 
-#endif /* SYSTEM_H_ */
+#if defined(__s390x__) && defined(__clang__)
+#define TLS_FIX_S390X
+#define TLS_FIX_S390X_MOVE(x) x
+#else
+#define TLS_FIX_S390X thread_local
+#define TLS_FIX_S390X_MOVE(x) std::move(x)
+#endif
\ No newline at end of file
